     /**
      * \brief Stream File Input Provider
      *
      * This input provider returns the data from a stream file block by block.
      * <br/>
      * The user data contains the absolute path to the IFS file.
      * 
      * \author Mihael Schmidt
      * \date   09.01.2011
      *
      * \implements RNGIN - Input Provider
      */

      *---------------------------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of RPG Next Gen Input Providers (RNGIN).
      *
      * RNGIN is free software: you can redistribute it and/or modify
      * it under the terms of the GNU Lesser General Public License as published
      * by the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * RNGIN is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU Lesser General Public License for more details.
      *
      * You should have received a copy of the GNU Lesser General Public License
      * along with RNGIN.  If not, see <http://www.gnu.org/licenses/>.
      *
      *---------------------------------------------------------------------------------------------


     H nomain


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
      /copy 'streamfile2_input_provider_h.rpgle'
      /copy 'ifsio_h.rpgle'
      /copy 'errno_h.rpgle'
      /copy 'libc_h.rpgle'
      /copy 'message_h.rpgle'

      /if not defined (QUSEC)
      /define QUSEC
      /copy QSYSINC/QRPGLESRC,QUSEC
      /endif


      *-------------------------------------------------------------------------
      * Data structures
      *-------------------------------------------------------------------------
     D tmpl_communication_area...
     D                 DS                   qualified template
     D   filePath                  1024A
     D   fileHandle                  10I 0
     D   fileStatus                  10I 0

      /copy 'input_provider_t.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D INPUT_PROVIDER_ID...
     D                 C                   'rng_input_streamfile2_provider'
      *
     D FILE_STATUS_OPEN...
     D                 C                   1
     D FILE_STATUS_CLOSED...
     D                 C                   2

      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create input provider
      *
      * Creates the input provider.
      * <br/>
      * The path to the stream file read must be passed as a parameter.
      * 
      * \param Path to the stream file (null-terminated)
      *
      * \return Pointer to input provider 
      * 
      * \info The calling program must make sure to free the allocated resources
      *       with the procedure <em>rng_input_stmf2_finalize</em>.
      */
     P rng_input_stmf2_create...
     P                 B                   export
     D                 PI              *
     D   pUserData                     *   const options(*nopass)
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(ptr)
     D commArea        DS                  likeds(tmpl_communication_area)
     D                                     based(header.commArea)
      /free
       ptr = %alloc(%size(header));

       header.id = INPUT_PROVIDER_ID;
       header.proc_create = %paddr('rng_input_stmf2_create');
       header.proc_open = %paddr('rng_input_stmf2_open');
       header.proc_read = %paddr('rng_input_stmf2_read');
       header.proc_close = %paddr('rng_input_stmf2_close');
       header.proc_finalize = %paddr('rng_input_stmf2_finalize');

       if (%parms() = 1 or pUserdata <> *null);
         header.userdata = %alloc(strlen(pUserData));
         memcpy(header.userdata : pUserData : strlen(pUserData) + 1); // +1 um NULL mit zu kopieren
       else;
         msg_sendEscapeMessage('The input file must be passed as userdata.');
       endif;

       header.commArea = %alloc(%size(tmpl_communication_area));
       commArea.filePath = %str(header.userdata);
       commArea.fileHandle = -1;
       commArea.fileStatus = FILE_STATUS_CLOSED;

       return ptr;
      /end-free
     P                 E


     /**
      * \brief Dispose input provider
      *
      * Frees allocated memory.
      *
      * \param Pointer to input provider
      */
     P rng_input_stmf2_finalize...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D commArea        DS                  likeds(tmpl_communication_area)
     D                                     based(header.commArea)
      /free
       if (commArea.fileStatus = FILE_STATUS_OPEN);
         callp close(commArea.fileHandle);
       endif;

       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) inputProvider;
      /end-free
     P                 E


     /**
      * \brief Open file
      *
      * Opens the stream file. This procedure must be called prior to any 
      * read calls. Once opened the file must be closed with the 
      * <em>rng_input_stmf2_close</em> procedure.
      *
      * \param Pointer to input provider
      */
     P rng_input_stmf2_open...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D commArea        DS                  likeds(tmpl_communication_area)
     D                                     based(header.commArea)
     D mode            S             10I 0
     D errorNumber     S             10I 0 based(errorNumberPtr)
     D errorMessage    S            200A
      /free
       if (commArea.fileStatus <> FILE_STATUS_OPEN);
         mode = O_RDONLY + O_TEXTDATA;
         commArea.fileHandle = open(%trimr(commArea.filePath) : mode);

         if (commArea.fileHandle <> -1);
           commArea.fileStatus = FILE_STATUS_OPEN;
         else;
           errorNumberPtr = c__errno();
           errorMessage = %str(strerror(errorNumber));
           msg_sendEscapeMessage('Error on opening the file ' + 
                                 %trimr(commArea.filePath) +
                                 ' could not be opened. ' +
                                 %char(errorNumber)+' - '+%trimr(errorMessage));
         endif;

       endif;
      /end-free
     P                 E


     /**
      * \brief Close file
      *
      * Closes any open files opened by this input provider.
      * 
      * \param Pointer to input provider
      */
     P rng_input_stmf2_close...
     P                 B                   export
     D                 PI
     D   inputProvider...
     D                                 *   const
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D commArea        DS                  likeds(tmpl_communication_area)
     D                                     based(header.commArea)
      /free
       if (commArea.fileStatus <> FILE_STATUS_CLOSED);
         callp close(commArea.fileHandle);
         commArea.fileStatus = FILE_STATUS_CLOSED;
       endif;
      /end-free
     P                 E


     /**
      * \brief Read block from file
      *
      * The data will be read and returned blockwise from the file. 
      *
      * \param Input provider
      * \param Data read
      *
      * \return *on = EOF (end of file) else *off
      */
     P rng_input_stmf2_read...
     P                 B                   export
     D                 PI              N
     D   inputProvider...
     D                                 *   const
     D   data                     65535A   varying
      *
     D header          DS                  likeds(rng_input_provider_t)
     D                                     based(inputProvider)
     D commArea        DS                  likeds(tmpl_communication_area)
     D                                     based(header.commArea)
     D buffer          S          65535A
     D readLength      S             10I 0
      /free
       // load next block from file into buffer
       readLength = read(commArea.fileHandle : %addr(buffer) : %size(buffer));
       data = %subst(buffer : 1 : readLength);
       
       if (readLength < 1);
         return *off;
       else;
         return *on;
       endif;
      /end-free
     P                 E
